# -*- coding: utf-8 -*-
import os
import io
import paramiko
from jms_storage.base import ObjectStorage


class SFTPStorage(ObjectStorage):

    def __init__(self, config):
        self.sftp = None
        self.sftp_host = config.get('SFTP_HOST', None)
        self.sftp_port = int(config.get('SFTP_PORT', 22))
        self.sftp_username = config.get('SFTP_USERNAME', '')
        self.sftp_secret_type = config.get('STP_SECRET_TYPE', 'password')
        self.sftp_password = config.get('SFTP_PASSWORD', '')
        self.sftp_private_key = config.get('STP_PRIVATE_KEY', '')
        self.sftp_passphrase = config.get('STP_PASSPHRASE', '')
        self.sftp_root_path = config.get('SFTP_ROOT_PATH', '/tmp')
        self.ssh = paramiko.SSHClient()
        self.connect()

    def connect(self):
        self.ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        if self.sftp_secret_type == 'password':
            self.ssh.connect(self.sftp_host, self.sftp_port, self.sftp_username, self.sftp_password)
        elif self.sftp_secret_type == 'ssh_key':
            pkey = paramiko.RSAKey.from_private_key(io.StringIO(self.sftp_private_key))
            self.ssh.connect(self.sftp_host, self.sftp_port, self.sftp_username, pkey=pkey,
                             passphrase=self.sftp_passphrase)
        self.sftp = self.ssh.open_sftp()

    def confirm_connected(self):
        try:
            self.sftp.getcwd()
        except Exception as e:
            self.connect()

    def upload(self, src, target):
        local_file = src
        remote_file = os.path.join(self.sftp_root_path, target)
        try:
            self.confirm_connected()
            mode = os.stat(local_file).st_mode
            remote_dir = os.path.dirname(remote_file)
            if not self.exists(remote_dir):
                self.sftp.mkdir(remote_dir)
            self.sftp.put(local_file, remote_file)
            self.sftp.chmod(remote_file, mode)
            return True, None
        except Exception as e:
            return False, e

    def download(self, src, target):
        remote_file = src
        local_file = target
        self.confirm_connected()
        try:
            local_dir = os.path.dirname(local_file)
            if not os.path.exists(local_dir):
                os.makedirs(local_dir)
            mode = self.sftp.stat(remote_file).st_mode
            self.sftp.get(remote_file, local_file)
            os.chmod(local_file, mode)
            return True, None
        except Exception as e:
            return False, e

    def delete(self, path):
        path = os.path.join(self.sftp_root_path, path)
        self.confirm_connected()
        if not self.exists(path):
            raise FileNotFoundError('File not exist error(%s)' % path)
        try:
            self.sftp.remove(path)
            return True, None
        except Exception as e:
            return False, e

    def check_dir_exist(self, d):
        self.confirm_connected()
        try:
            self.sftp.stat(d)
            return True
        except Exception:
            return False

    def mkdir(self, dirs):
        self.confirm_connected()
        try:
            if not self.exists(dirs):
                self.sftp.mkdir(dirs)
            return True, None
        except Exception as e:
            return False, e

    def exists(self, target):
        self.confirm_connected()
        try:
            self.sftp.stat(target)
            return True
        except:
            return False

    def close(self):
        self.sftp.close()
        self.ssh.close()



# -*- coding: utf-8 -*-
#
import os

import oss2

from .base import ObjectStorage


class OSSStorage(ObjectStorage):
    def __init__(self, config):
        self.endpoint = config.get("ENDPOINT", None)
        self.bucket = config.get("BUCKET", None)
        self.access_key = config.get("ACCESS_KEY", None)
        self.secret_key = config.get("SECRET_KEY", None)
        if self.access_key and self.secret_key:
            self.auth = oss2.Auth(self.access_key, self.secret_key)
        else:
            self.auth = None
        if self.auth and self.endpoint and self.bucket:
            self.client = oss2.Bucket(self.auth, self.endpoint, self.bucket)
        else:
            self.client = None

    def upload(self, src, target):
        try:
            self.client.put_object_from_file(target, src)
            return True, None
        except Exception as e:
            return False, e

    def exists(self, path):
        return self.client.object_exists(path)

    def delete(self, path):
        try:
            self.client.delete_object(path)
            return True, None
        except Exception as e:
            return False, e

    def download(self, src, target):
        try:
            os.makedirs(os.path.dirname(target), 0o755, exist_ok=True)
            self.client.get_object_to_file(src, target)
            return True, None
        except Exception as e:
            return False, e

    def list_buckets(self):
        service = oss2.Service(self.auth,self.endpoint)
        return ([b.name for b in oss2.BucketIterator(service)])

    @property
    def type(self):
        return 'oss'



# -*- coding: utf-8 -*-
#

import os
from ftplib import FTP, error_perm
from .base import ObjectStorage


class FTPStorage(ObjectStorage):

    def __init__(self, config):
        self.host = config.get("HOST", None)
        self.port = int(config.get("PORT", 21))
        self.username = config.get("USERNAME", None)
        self.password = config.get("PASSWORD", None)
        self.pasv = bool(config.get("PASV", False))
        self.dir = config.get("DIR", "replay")
        self.client = FTP()
        self.client.encoding = 'utf-8'
        self.client.set_pasv(self.pasv)
        self.pwd = '.'
        self.connect()

    def connect(self, timeout=-999, source_address=None):
        self.client.connect(self.host, self.port, timeout, source_address)
        self.client.login(self.username, self.password)
        if not self.check_dir_exist(self.dir):
            self.mkdir(self.dir)
        self.client.cwd(self.dir)
        self.pwd = self.client.pwd()

    def confirm_connected(self):
        try:
            self.client.pwd()
        except Exception:
            self.connect()

    def upload(self, src, target):
        self.confirm_connected()
        target_dir = os.path.dirname(target)
        exist = self.check_dir_exist(target_dir)
        if not exist:
            ok = self.mkdir(target_dir)
            if not ok:
                raise PermissionError('Dir create error: %s' % target)
        try:
            with open(src, 'rb') as f:
                self.client.storbinary('STOR '+target, f)
            return True, None
        except Exception as e:
            return False, e

    def download(self, src, target):
        self.confirm_connected()
        try:
            os.makedirs(os.path.dirname(target), 0o755, exist_ok=True)
            with open(target, 'wb') as f:
                self.client.retrbinary('RETR ' + src, f.write)
            return True, None
        except Exception as e:
            return False, e

    def delete(self, path):
        self.confirm_connected()
        if not self.exists(path):
            raise FileNotFoundError('File not exist error(%s)' % path)
        try:
            self.client.delete(path)
            return True, None
        except Exception as e:
            return False, e

    def check_dir_exist(self, d):
        pwd = self.client.pwd()
        try:
            self.client.cwd(d)
            self.client.cwd(pwd)
            return True
        except error_perm:
            return False

    def mkdir(self, dirs):
        self.confirm_connected()
        # 创建多级目录，ftplib不支持一次创建多级目录
        dir_list = dirs.split('/')
        pwd = self.client.pwd()
        try:
            for d in dir_list:
                if not d or d in ['.']:
                    continue
                # 尝试切换目录
                try:
                    self.client.cwd(d)
                    continue
                except:
                    pass
                # 切换失败创建这个目录，再切换
                try:
                    self.client.mkd(d)
                    self.client.cwd(d)
                except:
                    return False
            return True
        finally:
            self.client.cwd(pwd)

    def exists(self, target):
        self.confirm_connected()
        try:
            self.client.size(target)
            return True
        except:
            return False

    def close(self):
        self.client.close()



# -*- coding: utf-8 -*-
#

import os

from azure.storage.blob import BlobServiceClient

from .base import ObjectStorage


class AzureStorage(ObjectStorage):

    def __init__(self, config):
        self.account_name = config.get("ACCOUNT_NAME", None)
        self.account_key = config.get("ACCOUNT_KEY", None)
        self.container_name = config.get("CONTAINER_NAME", None)
        self.endpoint_suffix = config.get("ENDPOINT_SUFFIX", 'core.chinacloudapi.cn')

        if self.account_name and self.account_key:
            self.service_client = BlobServiceClient(
                account_url=f'https://{self.account_name}.blob.{self.endpoint_suffix}',
                credential={'account_name': self.account_name, 'account_key': self.account_key}
            )
            self.client = self.service_client.get_container_client(self.container_name)
        else:
            self.client = None

    def upload(self, src, target):
        try:
            self.client.upload_blob(target, src)
            return True, None
        except Exception as e:
            return False, e

    def download(self, src, target):
        try:
            blob_data = self.client.download_blob(blob=src)
            os.makedirs(os.path.dirname(target), 0o755, exist_ok=True)
            with open(target, 'wb') as writer:
                writer.write(blob_data.readall())
            return True, None
        except Exception as e:
            return False, e

    def delete(self, path):
        try:
            self.client.delete_blob(path)
            return True, False
        except Exception as e:
            return False, e

    def exists(self, path):
        resp = self.client.list_blobs(name_starts_with=path)
        return len(list(resp)) != 0

    def list_buckets(self):
        return list(self.service_client.list_containers())

    @property
    def type(self):
        return 'azure'
